//
// Night rider code for 15 LEDs
// Here the 'sweep' is done with different PWM values  for the LED's
// So you really get the 'fading' light beam.
// Originally written in 2004.
//
// Freeware 2018, Fen Logic Ltd.
// This code is free and is delivered 'as is'.
// It comes without any warranty, to the extent permitted by applicable law.
// There are no restrictions to any use or re-use of this code
// in any form or shape. It would be nice if you keep my company name
// in the source or modified source code but even that is not
// required as I can't check it anyway.
// But the code comes with no warranties or guarantees whatsoever.
// 
// The code is very straightforward so with a little bit of effort 
// it can be adapted to a different number of LEDs.
// As the light intensity depends greatly on the LED type
// the PWM value are controlled by parameters.
// 

module nightrider 
#(parameter CLOCK_FREQ = 10000000)
(
   input  reset_n,
   input  clk,
   
   output reg [14:0] led
);

// Intensity depends on the LED type used
// This is a linear scale in 5 steps 
// Adapt as required 
localparam PWM1_INCR = 9'd051;
localparam PWM2_INCR = 9'd102;
localparam PWM3_INCR = 9'd153;
localparam PWM4_INCR = 9'd204;

// Sweep in ~1 sec over 15 position
localparam SEQ_PRESCALE = CLOCK_FREQ/15 ;
// PWM LEDs run at higher freq. to prevent 'flicker' 
localparam PWM_PRESCALE = CLOCK_FREQ/5000 ; 

reg [ 8:0] pwm1_count;
reg [ 8:0] pwm2_count;
reg [ 8:0] pwm3_count;
reg [ 8:0] pwm4_count;
reg [ 3:0] seq; 
reg        down;
reg [15:0] pwm_prescale; // Enough for clock up to ~250 MHz
reg [23:0] seq_prescale; // Enough for clock up to ~250 MHz

wire [3:0] pwm;

   always @(posedge clk or negedge reset_n)
   begin
      if (!reset_n)
      begin
         pwm1_count   <= 9'h000;
         pwm2_count   <= 9'h000;
         pwm3_count   <= 9'h000;
         pwm4_count   <= 9'h000;
         seq          <= 4'h0;
         down         <= 1'b0;
         pwm_prescale <= 16'h0; 
         seq_prescale <= 20'h0; 
      end
      else
      begin
         // Generate 4 different PWM signals 
         // Uses a 16-bit prescaler
         if (pwm_prescale == PWM_PRESCALE)
         begin
            pwm_prescale <= 16'h0;
            pwm1_count <= (pwm1_count & 9'h0FF) + PWM1_INCR;
            pwm2_count <= (pwm2_count & 9'h0FF) + PWM2_INCR;
            pwm3_count <= (pwm3_count & 9'h0FF) + PWM3_INCR;
            pwm4_count <= (pwm4_count & 9'h0FF) + PWM4_INCR;
         end
         else
            pwm_prescale <= pwm_prescale + 16'h1;

         // This is the 15-pattern control counter.
         // With a 24-bit prescaler
         // It counts up and then down again 
         if (seq_prescale==SEQ_PRESCALE)
         begin
            seq_prescale <= 24'h0;
            // Next pattern 
            if (down)
            begin
               if (seq==4'd1)
                  down <= 1'b0;
               seq <= seq - 4'h1;
            end
            else
            begin
               if (seq==4'd13)
                  down <= 1'b1;
               seq <= seq + 4'h1;
            end
         end
         else
            seq_prescale <= seq_prescale + 24'h1;
         
      end
   end
   
   // The four PWM signals.
   assign pwm[0] = pwm1_count[8];
   assign pwm[1] = pwm2_count[8];
   assign pwm[2] = pwm3_count[8];
   assign pwm[3] = pwm4_count[8];

   
   // Assign an intensity to each LED for each of the 15 patterns
   // From max. intensity one side to max. intensity other side 
   // For opposite 'movement' the sequence is counted down
   always @( * )
   begin
      case (seq)
      4'h0 : led = {   1'b1, pwm[3], pwm[2], pwm[1], pwm[0],                                        10'b0};
      4'h1 : led = { pwm[3],   1'b1, pwm[3], pwm[2], pwm[1], pwm[0],                                 9'b0};
      4'h2 : led = { pwm[2], pwm[3],   1'b1, pwm[3], pwm[2], pwm[1], pwm[0],                         8'b0};
      4'h3 : led = { pwm[1], pwm[2], pwm[3],   1'b1, pwm[3], pwm[2], pwm[1], pwm[0],                 7'b0};
      4'h4 : led = { pwm[0], pwm[1], pwm[2], pwm[3],   1'b1, pwm[3], pwm[2], pwm[1], pwm[0],         6'b0};
      4'h5 : led = {   1'b0, pwm[0], pwm[1], pwm[2], pwm[3],   1'b1, pwm[3], pwm[2], pwm[1], pwm[0], 5'b0};
      4'h6 : led = {   2'b0, pwm[0], pwm[1], pwm[2], pwm[3],   1'b1, pwm[3], pwm[2], pwm[1], pwm[0], 4'b0};
      4'h7 : led = {   3'b0, pwm[0], pwm[1], pwm[2], pwm[3],   1'b1, pwm[3], pwm[2], pwm[1], pwm[0], 3'b0};
      4'h8 : led = {   4'b0, pwm[0], pwm[1], pwm[2], pwm[3],   1'b1, pwm[3], pwm[2], pwm[1], pwm[0], 2'b0};
      4'h9 : led = {   5'b0, pwm[0], pwm[1], pwm[2], pwm[3],   1'b1, pwm[3], pwm[2], pwm[1], pwm[0], 1'b0};
      4'hA : led = {   6'b0, pwm[0], pwm[1], pwm[2], pwm[3],   1'b1, pwm[3], pwm[2], pwm[1], pwm[0]      };
      4'hB : led = {   7'b0, pwm[0], pwm[1], pwm[2], pwm[3],   1'b1, pwm[3], pwm[2], pwm[1]              };
      4'hC : led = {   8'b0, pwm[0], pwm[1], pwm[2], pwm[3],   1'b1, pwm[3], pwm[2]                      };
      4'hD : led = {   9'b0, pwm[0], pwm[1], pwm[2], pwm[3],   1'b1, pwm[3]                              };
      4'hE : led = {  10'b0, pwm[0], pwm[1], pwm[2], pwm[3],   1'b1                                      };
      default : led =  16'b0; 
      endcase
   end
   
endmodule   
   
   
   
   



 
